//--------------------------------------------------------------------------------------
//  Plik I1O1W2.cpp
//  Autor: Piotr M. Szczypinski
//  Instytut Elektroniki Politechniki Ldzkiej
//--------------------------------------------------------------------------------------

#include <streams.h>    
#include <initguid.h>
#include "I1O1W2.h"    

//--------------------------------------------------------------------------------------
//!!! Numer CLSID/GUID filtru wygenerowany przez program GuidGen
// {C5881294-2485-49c2-BDAA-166C0139A4C9}
DEFINE_GUID(CLSID_I1O1W2, 
			0xc5881294, 0x2485, 0x49c2, 0xbd, 0xaa, 0x16, 0x6c, 0x1, 0x39, 0xa4, 0xc9);


//--------------------------------------------------------------------------------------
// Dane potrzebne do utworzenia filtru i jego wyprowadzen
const AMOVIESETUP_MEDIATYPE
sudPinTypes =   { &MEDIATYPE_NULL                // clsMajorType
, &MEDIASUBTYPE_NULL }  ;       // clsMinorType

const AMOVIESETUP_PIN
psudPins[] = { { L"Input"            // strName
, FALSE               // bRendered
, FALSE               // bOutput
, FALSE               // bZero
, FALSE               // bMany
, &CLSID_NULL         // clsConnectsToFilter
, L"Output"           // strConnectsToPin
, 1                   // nTypes
, &sudPinTypes }      // lpTypes
, { L"Output"           // strName
, FALSE               // bRendered
, TRUE                // bOutput
, FALSE               // bZero
, FALSE               // bMany
, &CLSID_NULL         // clsConnectsToFilter
, L"Input"            // strConnectsToPin
, 1                   // nTypes
, &sudPinTypes } };   // lpTypes


const AMOVIESETUP_FILTER
sudNullIP = { &CLSID_I1O1W2                  // clsID
, L"Wielordzenie_I1O1W2"                    // strName
, MERIT_DO_NOT_USE                // dwMerit
, 2                               // nPins
, psudPins };                     // lpPin

//--------------------------------------------------------------------------------------
// Potrzebne podczas wykonania CreateInstance
CFactoryTemplate g_Templates[]=
{   {L"Wielordzenie_I1O1W2"
, &CLSID_I1O1W2
, KlasaFiltru_I1O1W2::CreateInstance
, NULL
, &sudNullIP }
};
int g_cTemplates = sizeof(g_Templates)/sizeof(g_Templates[0]);

//--------------------------------------------------------------------------------------
// Konstuktor klasy filtru
KlasaFiltru_I1O1W2::KlasaFiltru_I1O1W2(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr)
: CTransformFilter (tszName, punk, CLSID_I1O1W2)
{
} 

//--------------------------------------------------------------------------------------
// Tworzy obiekt COM filtru Zielenina
CUnknown * WINAPI KlasaFiltru_I1O1W2::CreateInstance(LPUNKNOWN punk, HRESULT *phr) 
{
	ASSERT(phr);

	KlasaFiltru_I1O1W2 *pNewObject = new KlasaFiltru_I1O1W2(NAME("Wielordzenie_I1O1W2"), punk, phr );
	if(pNewObject == NULL) 
		if(phr)
			*phr = E_OUTOFMEMORY;

	return pNewObject;
}

HRESULT KlasaFiltru_I1O1W2::DecideBufferSize(IMemAllocator *pAlloc,ALLOCATOR_PROPERTIES *pProperties)
{
	if (m_pInput->IsConnected() == FALSE) {
		return E_UNEXPECTED;
	}
	CheckPointer(pAlloc,E_POINTER);
	CheckPointer(pProperties,E_POINTER);
	HRESULT hr = NOERROR;
	pProperties->cBuffers = 1;
	pProperties->cbBuffer = m_pInput->CurrentMediaType().GetSampleSize();
	ASSERT(pProperties->cbBuffer);
	ALLOCATOR_PROPERTIES Actual;
	hr = pAlloc->SetProperties(pProperties,&Actual);
	if (FAILED(hr)) {
		return hr;
	}
	ASSERT( Actual.cBuffers == 1 );
	if (pProperties->cBuffers > Actual.cBuffers ||
		pProperties->cbBuffer > Actual.cbBuffer) {
			return E_FAIL;
	}
	return NOERROR;
}

//------------------------------------------------------------------------------
//!!! Metoda informujaca o dostarczanych na wyjsciu rodzajach probek
HRESULT KlasaFiltru_I1O1W2::GetMediaType(int iPosition, CMediaType *pMediaType)
{
	if (m_pInput->IsConnected() == FALSE) return E_UNEXPECTED;
	if (iPosition < 0) return E_INVALIDARG;
	if (iPosition > 0) return VFW_S_NO_MORE_ITEMS;
	CheckPointer(pMediaType,E_POINTER);
	*pMediaType = m_pInput->CurrentMediaType();
	return NOERROR;
}

//--------------------------------------------------------------------------------------
//!!! Funkcja wywolywana podczas sprawdzania zgodnosci formatow wejsciowego i wyjsciowego
HRESULT KlasaFiltru_I1O1W2::CheckTransform(const CMediaType *mtIn,const CMediaType *mtOut)
{
	CheckPointer(mtIn,E_POINTER);
	CheckPointer(mtOut,E_POINTER);

	if (IsEqualGUID(*mtIn->Type(), MEDIATYPE_Video)) 
	{
		if (IsEqualGUID(*mtIn->Subtype(), MEDIASUBTYPE_RGB24)) 
		{
			VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *) mtIn->Format();
			if (pvi->bmiHeader.biBitCount == 24)
			{
				if (*mtIn == *mtOut) 
				{
					return NOERROR;
				}
			}
		}
	}
	return E_FAIL;
}

//--------------------------------------------------------------------------------------
// Funkcja wywolywana podczas negocjacji rodzaju transmisji
HRESULT KlasaFiltru_I1O1W2::CheckInputType(const CMediaType* mtIn) 
{
	if(*mtIn->FormatType() != FORMAT_VideoInfo) return VFW_E_TYPE_NOT_ACCEPTED;
	if(!(IsEqualGUID(*mtIn->Type(), MEDIATYPE_Video))) return VFW_E_TYPE_NOT_ACCEPTED;

	if(IsEqualGUID(*mtIn->Subtype(), MEDIASUBTYPE_RGB24))
	{
		VIDEOINFO *pvi = (VIDEOINFO *) mtIn->Format();
		if (pvi->bmiHeader.biBitCount != 24) 
		{	
			return VFW_E_TYPE_NOT_ACCEPTED;
		}
		return S_OK;
	} 
	return VFW_E_TYPE_NOT_ACCEPTED;
}

//--------------------------------------------------------------------------------------
// Funkcja wywolywana do transformowania probki (obrazu) 
// Zmiana, dwa argumenty, probka wejsciowa i wyjsciowa
HRESULT KlasaFiltru_I1O1W2::Transform(IMediaSample *pInSample, IMediaSample *pOutSample)
{
	BYTE *pInData;                
	BYTE *pOutData;                
	long lDataLen;              
	int x, y;  
	RGBTRIPLE *prgbIn;            
	RGBTRIPLE *prgbOut;            

	AM_MEDIA_TYPE* pType = &m_pInput->CurrentMediaType();
	VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *) pType->pbFormat;
	ASSERT(pvi);

	CheckPointer(pInSample,E_POINTER);
	pInSample->GetPointer(&pInData);

	CheckPointer(pOutSample,E_POINTER);
	pOutSample->GetPointer(&pOutData);

	lDataLen = pInSample->GetSize();
	if(lDataLen > pOutSample->GetSize()) lDataLen = pOutSample->GetSize();

	int width     = pvi->bmiHeader.biWidth;
	int height = abs(pvi->bmiHeader.biHeight);
	int bits     = pvi->bmiHeader.biBitCount;
	int stride     = (width * sizeof( RGBTRIPLE ) + 3) & -4;
	int pixels  = width * height;

	int left     = width / 4;
	int bottom = height /4;
	int right = width - left;
	int top = height - bottom;

	CopyMemory((PVOID)pOutData,(PVOID)pInData, lDataLen);

	for (y = bottom; y < top; y++)
	{
		for (x = left; x < right; x++) 
		{
			prgbIn = (RGBTRIPLE*) (pInData + stride * y + sizeof(RGBTRIPLE)*x);
			prgbOut = (RGBTRIPLE*) (pOutData + stride * y + sizeof(RGBTRIPLE)*x);
			prgbOut->rgbtGreen = 255 - prgbIn->rgbtGreen;
			prgbOut->rgbtBlue = 255 - prgbIn->rgbtBlue;
			prgbOut->rgbtRed = 255 - prgbIn->rgbtRed;
		}
	}
	return NOERROR;
}

//--------------------------------------------------------------------------------------
// Funkcja otrzymuje probke (obraz) z wejscia
HRESULT KlasaFiltru_I1O1W2::Receive(IMediaSample *pSample)
{
	HRESULT hr;
	ASSERT(pSample);

	AM_SAMPLE2_PROPERTIES * const pProps = m_pInput->SampleProps();
	if (pProps->dwStreamId != AM_STREAM_MEDIA) return NOERROR;
	ASSERT (m_pOutput != NULL) ;

	IMediaSample * probka_DoWyslania;
	hr = InitializeOutputSample(pSample, &probka_DoWyslania);
	if (FAILED(hr)) return hr;

	if (!FAILED(hr)) 
	{
		if (hr == NOERROR) 
		{
			m_bSampleSkipped = FALSE;
			Transform(pSample, probka_DoWyslania);
			m_pOutput->Deliver(probka_DoWyslania);
			probka_DoWyslania->Release();
			return NOERROR;
		} 
		else 
		{
			if (S_FALSE == hr) 
			{
				probka_DoWyslania->Release();
				m_bSampleSkipped = TRUE;
				if (!m_bQualityChanged) 
				{
					NotifyEvent(EC_QUALITY_CHANGE,0,0);
					m_bQualityChanged = TRUE;
				}
				return NOERROR;
			}
		}
	}
	return hr;
}

//--------------------------------------------------------------------------------------
// Funkcje wywolywane podczas rejestracji filtru przez program RegSvr32
STDAPI DllRegisterServer()
{
	return AMovieDllRegisterServer2( TRUE );
}

STDAPI DllUnregisterServer()
{
	return AMovieDllRegisterServer2( FALSE );
}


//--------------------------------------------------------------------------------------
// Funkcje niezbedne w bibliotece DLL
extern "C" BOOL WINAPI DllEntryPoint(HINSTANCE, ULONG, LPVOID);

BOOL APIENTRY DllMain(HANDLE hModule, 
					  DWORD  dwReason, 
					  LPVOID lpReserved)
{
	return DllEntryPoint((HINSTANCE)(hModule), dwReason, lpReserved);
}


